@@ -774,18 +774,18 @@ DECLARE
   in_priv TEXT;
   out_bits BIT(24);
   i INT;
-  all BIT(24);
+  allprivs BIT(24);
   start INT;
   finish INT;
 BEGIN
   out_bits := 0::BIT(24);
-  all := ~ out_bits;
+  allprivs := ~ out_bits;
   SELECT array_lower(raw_privs,1) INTO start;
   SELECT array_upper(raw_privs,1) INTO finish;
   FOR i IN start .. finish  LOOP
     SELECT out_bits | privilege_to_bits(raw_privs[i]) INTO out_bits;
-    IF out_bits = all THEN
-      RETURN all;
+    IF out_bits = allprivs THEN
+      RETURN allprivs;
     END IF;
   END LOOP;
   RETURN out_bits;
@@ -1193,7 +1193,7 @@ BEGIN
   IF NOT FOUND THEN
     RETURN FALSE;
   END IF;
-  SELECT dav_id INTO tmp_int FROM calendar_item WHERE dav_name = in_dav_name;
+  SELECT dav_id INTO tmp_int FROM caldav_data WHERE dav_name = in_dav_name;
   INSERT INTO sync_changes ( collection_id, sync_status, dav_id, dav_name)
                      VALUES( in_collection_id, in_status, tmp_int, in_dav_name);
   RETURN TRUE;
@@ -1213,7 +1213,8 @@ BEGIN
               AND sync_time > (SELECT modification_time FROM sync_tokens WHERE sync_token = in_old_sync_token)
             LIMIT 1;
     IF NOT FOUND THEN
-      RETURN in_old_sync_token;
+      -- They are in an inconsistent state: we return NULL so they can re-start the process
+      RETURN NULL;
     END IF;
   END IF;
   SELECT nextval('sync_tokens_sync_token_seq') INTO tmp_int;
