@@ -32,19 +32,19 @@ export AWL_DBAUSER=davical_dba
 export AWL_APPUSER=davical_app
 
 # Get the major version for PostgreSQL
-export DBVERSION="`psql -qAt -c "SELECT version();" template1 | cut -f2 -d' ' | cut -f1-2 -d'.'`"
+export DBVERSION="`psql -qXAt -c "SELECT version();" template1 | cut -f2 -d' ' | cut -f1-2 -d'.'`"
 
 install_note() {
   cat >>"${INSTALL_NOTE_FN}"
 }
 
 db_users() {
-  psql -qAt -c "SELECT usename FROM pg_user;" template1
+  psql -qXAt -c "SELECT usename FROM pg_user;" template1
 }
 
 create_db_user() {
   if ! db_users | grep "^${1}$" >/dev/null ; then
-    psql -qAt -c "CREATE USER ${1} NOCREATEDB NOCREATEROLE;" template1
+    psql -qXAt -c "CREATE USER ${1} NOCREATEDB NOCREATEROLE;" template1
     cat <<EONOTE | install_note
 *  You will need to edit the PostgreSQL pg_hba.conf to allow the
    '${1}' database user access to the 'davical' database.
@@ -54,15 +54,17 @@ EONOTE
 }
 
 create_plpgsql_language() {
-  if ! psql ${DBA} -qAt -c "SELECT lanname FROM pg_language;" "${DBNAME}" | grep "^plpgsql$" >/dev/null; then
+  if ! psql ${DBA} -qXAt -c "SELECT lanname FROM pg_language;" "${DBNAME}" | grep "^plpgsql$" >/dev/null; then
     createlang plpgsql "${DBNAME}"
   fi
 }
 
 try_db_user() {
-  [ "XtestX`psql -U "${1}" -qAt -c \"SELECT usename FROM pg_user;\" \"${DBNAME}\" 2>/dev/null`" != "XtestX" ]
+  [ "XtestX`psql -U "${1}" -qXAt -c \"SELECT usename FROM pg_user;\" \"${DBNAME}\" 2>/dev/null`" != "XtestX" ]
 }
 
+# Hide all the annoying NOTICE... messages
+export PGOPTIONS='--client-min-messages=warning'
 
 create_db_user "${AWL_DBAUSER}"
 create_db_user "${AWL_APPUSER}"
@@ -113,28 +115,33 @@ create_plpgsql_language
 
 #
 # Load the AWL base tables and schema management tables
-psql -qAt ${DBA} -f "${AWLDIR}/dba/awl-tables.sql" "${DBNAME}" 2>&1 | egrep -v "(^CREATE |^GRANT|^BEGIN|^COMMIT| NOTICE: )"
-psql -qAt ${DBA} -f "${AWLDIR}/dba/schema-management.sql" "${DBNAME}" 2>&1 | egrep -v "(^CREATE |^GRANT|^BEGIN|^COMMIT| NOTICE: |^t$)"
+psql -qXAt ${DBA} -f "${AWLDIR}/dba/awl-tables.sql" "${DBNAME}" 2>&1
+psql -qXAt ${DBA} -f "${AWLDIR}/dba/schema-management.sql" "${DBNAME}" 2>&1
 
 #
 # Load the DAViCal tables
-psql -qAt ${DBA} -f "${DBADIR}/davical.sql" "${DBNAME}" 2>&1 | egrep -v "(^CREATE |^GRANT|^BEGIN|^COMMIT| NOTICE: |^t$)"
+psql -qXAt ${DBA} -f "${DBADIR}/davical.sql" "${DBNAME}" 2>&1
 
 #
 # Set permissions for the application DB user on the database
 if ! ${DBADIR}/update-davical-database --dbname "${DBNAME}" --appuser "${AWL_APPUSER}" --nopatch --owner "${AWL_DBAUSER}" ; then
         cat <<EOFAILURE
 * * * * ERROR * * * *
-The database administration utility failed.  This is usually due to the Perl YAML
-or the Perl DBD::Pg libraries not being available.
+The database administration utility failed.  This may be due to database
+permissions for the davical_dba user, or because the Perl DBD::Pg or YAML
+libraries are not available.
 
-See:  http://wiki.davical.org/w/Install_Errors/No_Perl_YAML
+Check that your pg_hba.conf allows the davical_dba user to connect to the
+database (and make sure you've reloaded PostgreSQL since changing that).
+
+Also see:  http://wiki.davical.org/w/Install_Errors/No_Perl_YAML
 
 EOFAILURE
+  exit 1
 fi
 #
 # Load the required base data
-psql -qAt ${DBA} -f "${DBADIR}/base-data.sql" "${DBNAME}" | egrep -v '^10'
+psql -qXAt ${DBA} -f "${DBADIR}/base-data.sql" "${DBNAME}" | egrep -v '^10'
 
 #
 # We can override the admin password generation for regression testing predictability
@@ -158,7 +165,7 @@ if [ "$ADMINPW" = "" ] ; then
   ADMINPW="please change this password"
 fi
 
-psql -q -c "UPDATE usr SET password = '**${ADMINPW}' WHERE user_no = 1;" "${DBNAME}"
+psql -qX -c "UPDATE usr SET password = '**${ADMINPW}' WHERE user_no = 1;" "${DBNAME}"
 
 echo "NOTE"
 echo "===="
