@@ -39,6 +39,7 @@ $c->permission_scan_depth = 2;
 $c->expand_pdo_parameters = true;
 $c->home_calendar_name = 'home';
 $c->enable_row_linking = true;
+$c->enable_scheduling = false;
 $c->http_auth_mode = 'Basic';
 // $c->default_locale = array('es_MX', 'es_AR', 'es', 'pt');  // An array of locales to try, or just a single locale
 // $c->local_tzid = 'Pacific/Auckland';  // Perhaps we should read from /etc/timezone - I wonder how standard that is?
@@ -53,7 +54,7 @@ $c->images      = $c->base_url . '/images';
 
 // Add a default for newly created users
 $c->template_usr = array( 'active' => true,
-                          'locale' => 'en_EN',
+                          'locale' => 'en_GB',
                           'date_format_type' => 'E',
                           'email_ok' => date('Y-m-d')
                         );
@@ -115,6 +116,7 @@ $c->protocol_server_port_script = $c->protocol_server_port . ($_SERVER['SCRIPT_N
 * access which could break DAViCal completely by causing output to start
 * too early.
 */
+ob_start( );
 if ( @file_exists('/etc/davical/'.$_SERVER['SERVER_NAME'].'-conf.php') ) {
   include('/etc/davical/'.$_SERVER['SERVER_NAME'].'-conf.php');
 }
@@ -137,6 +139,8 @@ else {
   include('davical_configuration_missing.php');
   exit;
 }
+$config_warnings = trim(ob_get_contents());
+ob_end_clean();
 
 if ( !isset($c->page_title) ) $c->page_title = $c->system_name;
 
@@ -162,7 +166,7 @@ init_gettext( 'davical', $c->locale_path );
 *
 */
 $c->code_version = 0;
-$c->version_string = '0.9.9'; // The actual version # is replaced into that during the build /release process
+$c->version_string = '0.9.9.3'; // The actual version # is replaced into that during the build /release process
 if ( isset($c->version_string) && preg_match( '/(\d+)\.(\d+)\.(\d+)(.*)/', $c->version_string, $matches) ) {
   $c->code_major = $matches[1];
   $c->code_minor = $matches[2];
@@ -179,7 +183,7 @@ $_SERVER['SERVER_NAME'] = $c->domain_name;
 
 require_once('AwlQuery.php');
 
-$c->want_dbversion = array(1,2,8);
+$c->want_dbversion = array(1,2,9);
 $c->schema_version = 0;
 $qry = new AwlQuery( 'SELECT schema_major, schema_minor, schema_patch FROM awl_db_revision ORDER BY schema_id DESC LIMIT 1;' );
 if ( $qry->Exec('always',__LINE__,__FILE__) && $row = $qry->Fetch() ) {
@@ -241,6 +245,24 @@ function getUserByName( $username, $use_cache = true ) {
 }
 
 
+/**
+* Return a user record identified by e-mail address, caching it for any subsequent lookup
+* @param string $email The email address of the user record to retrieve
+* @param boolean $use_cache Whether or not to use the cache (default: yes)
+*/
+function getUserByEMail( $email, $use_cache = true ) {
+  global $_known_users_name;
+
+  if ( $use_cache ) {
+    /** We don't actually maintain a cache-access by e-mail, since it's rare */
+    foreach( $_known_users_name AS $k => $v ) {
+      if ( strtolower($email) == strtolower($v->email) ) return $v;
+    }
+  }
+  return _davical_get_principal_query_cached( 'lower(email) = lower(:param)', $email );
+}
+
+
 /**
 * Return a user record identified by a user_no, caching it for any subsequent lookup
 * @param int $user_no The ID of the record to retrieve
@@ -409,6 +431,7 @@ function DateToISODate( $indate ) {
 * @return integer A bit mask of the privileges.
 */
 define("DAVICAL_MAXPRIV", "65535");
+define("DAVICAL_ADDRESSBOOK_MAXPRIV", "1023");
 function privilege_to_bits( $raw_privs ) {
   $out_priv = 0;
 
@@ -448,7 +471,7 @@ function privilege_to_bits( $raw_privs ) {
   }
 
   // 'all' will include future privileges
-  if ( $out_priv >= DAVICAL_MAXPRIV ) $out_priv = pow(2,24) - 1;
+  if ( ($out_priv & DAVICAL_MAXPRIV) >= DAVICAL_MAXPRIV ) $out_priv = pow(2,24) - 1;
   return $out_priv;
 }
 
@@ -459,7 +482,7 @@ function privilege_to_bits( $raw_privs ) {
 * @param integer $raw_bits A bit mask of the privileges.
 * @return mixed The string (or array of strings) of privilege names
 */
-function bits_to_privilege( $raw_bits ) {
+function bits_to_privilege( $raw_bits, $resourcetype = 'resource' ) {
   $out_priv = array();
 
   if ( is_string($raw_bits) ) {
@@ -473,7 +496,7 @@ function bits_to_privilege( $raw_bits ) {
   if ( ($raw_bits &  16) != 0 ) $out_priv[] = 'DAV::read-acl';
   if ( ($raw_bits &  32) != 0 ) $out_priv[] = 'DAV::read-current-user-privilege-set';
   if ( ($raw_bits & 256) != 0 ) $out_priv[] = 'DAV::write-acl';
-  if ( ($raw_bits & 512) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:read-free-busy';
+  if ( ($resourcetype == 'calendar' || $resourcetype == 'proxy') && ($raw_bits & 512) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:read-free-busy';
 
   if ( ($raw_bits & 198) != 0 ) {
     if ( ($raw_bits & 198) == 198 ) $out_priv[] = 'DAV::write';
@@ -483,14 +506,14 @@ function bits_to_privilege( $raw_bits ) {
     if ( ($raw_bits & 128) != 0 ) $out_priv[] = 'DAV::unbind';
   }
 
-  if ( ($raw_bits & 7168) != 0 ) {
+  if ( $resourcetype == 'schedule-inbox' && ($raw_bits & 7168) != 0 ) {
     if ( ($raw_bits & 7168) == 7168 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-deliver';
     if ( ($raw_bits & 1024) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-deliver-invite';
     if ( ($raw_bits & 2048) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-deliver-reply';
     if ( ($raw_bits & 4096) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-query-freebusy';
   }
 
-  if ( ($raw_bits & 57344) != 0 ) {
+  if ( $resourcetype == 'schedule-outbox' && ($raw_bits & 57344) != 0 ) {
     if ( ($raw_bits & 57344) == 57344 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-send';
     if ( ($raw_bits &  8192) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-send-invite';
     if ( ($raw_bits & 16384) != 0 ) $out_priv[] = 'urn:ietf:params:xml:ns:caldav:schedule-send-reply';
