@@ -19,23 +19,42 @@ if ( isset($_SERVER['PATH_INFO']) && preg_match( '{^(/favicon.ico|davical.css|(i
   }
   exit(0);
 }
+elseif ( isset($_SERVER['PATH_INFO']) && preg_match( '{^/\.well-known/(.+)$}', $_SERVER['PATH_INFO'], $matches ) ) {
+	require ('well-known.php');
+  exit(0);
+}
 require_once('./always.php');
 // dbg_error_log( 'caldav', ' User agent: %s', ((isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : 'Unfortunately Mulberry does not send a "User-agent" header with its requests :-(')) );
 // dbg_log_array( 'headers', '_SERVER', $_SERVER, true );
 require_once('HTTPAuthSession.php');
 $session = new HTTPAuthSession();
 
-/**
-* access-control is rfc3744, we do some of it, but no way to say that.
-* calendar-schedule is another one we do some of, but the spec is not final yet either.
-*/
-if ( isset($c->override_dav_header) ) {
-  $dav = $c->override_dav_header;
+function send_dav_header() {
+  global $c;
+
+  /**
+  * access-control is rfc3744, we do most of it, but no way to say that.
+  * calendar-schedule is another one we do most of, but the spec is not final yet either.
+  */
+  if ( isset($c->override_dav_header) ) {
+    $dav = $c->override_dav_header;
+  }
+  else {
+    /** hack to get around bugzilla #463392 - remove sometime after 2011-02-28 */
+    if ( isset($_SERVER['HTTP_USER_AGENT']) && preg_match( '{ Gecko/(20[01]\d[01]\d[0123]\d)(\d+)? }', $_SERVER['HTTP_USER_AGENT'], $matches ) && $matches[1] < 20100520 ) {
+      $dav = '1, 2, 3, access-control, calendar-access, calendar-schedule, extended-mkcol, calendar-proxy, bind, addressbook';
+    }
+    else {
+       // We don't actually do calendar-auto-schedule yet - when we do we should add it on here.
+      $dav = '1, 2, 3, access-control, calendar-access, calendar-schedule, extended-mkcol, calendar-proxy, bind, addressbook';
+    }
+  }
+  $dav = explode( "\n", wordwrap( $dav ) );
+  foreach( $dav AS $v ) {
+    header( 'DAV: '.trim($v, ', '), false);
+  }
 }
-else {
-  $dav = '1, 2, access-control, calendar-access, calendar-schedule, extended-mkcol, calendar-proxy, bind, calendar-auto-schedule';
-}
-header( 'DAV: '.$dav);
+send_dav_header();  // Avoid polluting global namespace
 
 require_once('CalDAVRequest.php');
 $request = new CalDAVRequest();
@@ -63,14 +82,16 @@ switch ( $request->method ) {
   case 'HEAD':       include('caldav-GET.php');            break;
   case 'PROPPATCH':  include('caldav-PROPPATCH.php');      break;
   case 'PUT':
+    $request->CoerceContentType();
     switch( $request->content_type ) {
       case 'text/calendar':
         /** use original DAViCal 'PUT' code which will be rewritten */
         include('caldav-PUT.php');
         break;
-/*      case 'text/vcard':
+      case 'text/vcard':
+      case 'text/x-vcard':
         include('caldav-PUT-vcard.php');
-        break;*/
+        break;
       default:
         include('caldav-PUT-default.php');
         break;
