@@ -33,6 +33,24 @@ function check_pdo_pgsql() {
   return isset($loaded_extensions['pdo_pgsql']);
 }
 
+function check_gettext() {
+  global $phpinfo, $loaded_extensions;
+
+  if ( !function_exists('gettext') ) return false;
+  return isset($loaded_extensions['gettext']);
+}
+
+function check_suhosin_server_strip() {
+  global $phpinfo, $loaded_extensions;
+
+  if ( !isset($loaded_extensions['suhosin']) ) return true;
+  if ( ini_get('suhosin.server.strip') == "0" ) {
+    return true;
+  }
+
+  return false;
+}
+
 include("interactive-page.php");
 include("page-header.php");
 
@@ -92,7 +110,9 @@ $dependencies = array(
   translate('DAViCal DB Schema version '). implode('.',$c->want_dbversion) => 'check_schema_version',
   translate('PHP PDO module available') => 'check_pdo',
   translate('PDO PostgreSQL drivers') => 'check_pdo_pgsql',
-  translate('PHP PostgreSQL available') => 'check_pgsql' /*,
+  translate('PHP PostgreSQL available') => 'check_pgsql',
+  translate('GNU gettext support') => 'check_gettext',
+  translate('Suhosin "server.strip" disabled') => 'check_suhosin_server_strip' /*,
   'YAML' => 'php5-syck' */
 );
 
@@ -131,6 +151,14 @@ $site_statistics_table = build_site_statistics();
 
 $heading_config_clients = translate('Configuring Calendar Clients for DAViCal');
 $heading_config_davical = translate('Configuring DAViCal');
+$davical_configuration_errors = '';
+if ( $config_warnings != '' ) {
+  $davical_configuration_errors = '<div class="error"><h3 class="error">' . translate('Your configuration produced PHP errors which should be corrected') . '</h3>
+<pre>
+'.$config_warnings.'
+</pre></div>
+';
+}
 
   echo <<<EOBODY
 <style>
@@ -209,6 +237,7 @@ to configure Evolution, Sunbird, Lightning and Mulberry to use remotely hosted c
 
 <h2>$heading_config_davical</h2>
 <p>If you can read this then things must be mostly working already.</p>
+$davical_configuration_errors
 <p>The <a href="http://www.davical.org/installation.php">installation page on the DAViCal website</a> has
 some further information on how to install and configure this application.</p>
 EOBODY;
