@@ -163,9 +163,18 @@ class CalDAVRequest
       $_SERVER['REQUEST_METHOD'] = $_SERVER['HTTP_X_HTTP_METHOD_OVERRIDE'];
     }
     $this->method = $_SERVER['REQUEST_METHOD'];
-    $this->content_type = (isset($_SERVER['CONTENT_TYPE']) ? $_SERVER['CONTENT_TYPE'] : null);
-    if ( preg_match( '{^(\S+/\S+)\s*(;.*)?$}', $this->content_type, $matches ) ) {
-      $this->content_type = $matches[1];
+    if ( isset($_SERVER['CONTENT_LENGTH']) && $_SERVER['CONTENT_LENGTH'] > 7 ) {
+      $this->content_type = (isset($_SERVER['CONTENT_TYPE']) ? $_SERVER['CONTENT_TYPE'] : null);
+      if ( preg_match( '{^(\S+/\S+)\s*(;.*)?$}', $this->content_type, $matches ) ) {
+        $this->content_type = $matches[1];
+      }
+      if ( $this->method == 'PROPFIND' || $this->method == 'REPORT' ) {
+        if ( !preg_match( '{^(text|application)/xml$}', $this->content_type ) ) {
+          dbg_error_log( "LOG request", 'Request is "%s" but client set content-type to "%s". Assuming they meant XML!',
+                                                 $request->method, $this->content_type );
+          $this->content_type = 'text/xml';
+        }
+      }
     }
     $this->user_agent = ((isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : "Probably Mulberry"));
 
@@ -226,7 +235,7 @@ class CalDAVRequest
     * LOCK things use a "Timeout" header to set a series of reducing alternative values
     */
     if ( isset($_SERVER['HTTP_TIMEOUT']) ) {
-      $timeouts = split( ',', $_SERVER['HTTP_TIMEOUT'] );
+      $timeouts = explode( ',', $_SERVER['HTTP_TIMEOUT'] );
       foreach( $timeouts AS $k => $v ) {
         if ( strtolower($v) == 'infinite' ) {
           $this->timeout = (isset($c->maximum_lock_timeout) ? $c->maximum_lock_timeout : 86400 * 100);
@@ -356,7 +365,6 @@ EOSQL;
       $this->collection_type = 'email';
       $this->collection_path = $this->path;
       $this->_is_principal = true;
-//      $this->by_email = true;
     }
     else if ( preg_match( '#^(/[^/]+)/?$#', $this->path, $matches) || preg_match( '#^(/principals/[^/]+/[^/]+)/?$#', $this->path, $matches) ) {
       $this->collection_id = -1;
@@ -407,6 +415,7 @@ EOSQL;
                             'dav_name' => '/',
                             'dav_etag' => md5($c->system_name),
                             'is_calendar' => 'f',
+                            'is_addressbook' => 'f',
                             'is_principal' => 'f',
                             'user_no' => 0,
                             'dav_displayname' => $c->system_name,
@@ -493,6 +502,15 @@ EOSQL;
         )
       );
     }
+    if ( isset($this->collection) && $this->collection->is_addressbook ) {
+      $this->supported_reports = array_merge(
+        $this->supported_reports,
+        array(
+          'urn:ietf:params:xml:ns:carddav:addressbook-query' => '',
+          'urn:ietf:params:xml:ns:carddav:addressbook-multiget' => ''
+        )
+      );
+    }
 
 
     /**
@@ -525,11 +543,11 @@ EOSQL;
     * Look out for If-None-Match or If-Match headers
     */
     if ( isset($_SERVER["HTTP_IF_NONE_MATCH"]) ) {
-      $this->etag_none_match = str_replace('"','',$_SERVER["HTTP_IF_NONE_MATCH"]);
+      $this->etag_none_match = $_SERVER["HTTP_IF_NONE_MATCH"];
       if ( $this->etag_none_match == '' ) unset($this->etag_none_match);
     }
     if ( isset($_SERVER["HTTP_IF_MATCH"]) ) {
-      $this->etag_if_match = str_replace('"','',$_SERVER["HTTP_IF_MATCH"]);
+      $this->etag_if_match = $_SERVER["HTTP_IF_MATCH"];
       if ( $this->etag_if_match == '' ) unset($this->etag_if_match);
     }
   }
@@ -833,6 +851,42 @@ EOSQL;
   }
 
 
+  /**
+  * Coerces the Content-type of the request into something valid/appropriate
+  */
+  function CoerceContentType() {
+    if ( isset($this->content_type) ) {
+      $type = explode( '/', $this->content_type, 2);
+      /** @todo: Perhaps we should look at the target collection type, also. */
+      if ( $type[0] == 'text' ) return;
+    }
+
+    /** Null (or peculiar) content-type supplied so we have to try and work it out... */
+    $first_word = trim(substr( $this->raw_post, 0, 30));
+    $first_word = strtoupper( preg_replace( '/\s.*/s', '', $first_word ) );
+    switch( $first_word ) {
+      case '<?XML':
+        dbg_error_log( 'LOG WARNING', 'Application sent content-type of "%s" instead of "text/xml"',
+                                        (isset($this->content_type)?$this->content_type:'(null)') );
+        $this->content_type = 'text/xml';
+        break;
+      case 'BEGIN:VCALENDAR':
+        dbg_error_log( 'LOG WARNING', 'Application sent content-type of "%s" instead of "text/calendar"',
+                                        (isset($this->content_type)?$this->content_type:'(null)') );
+        $this->content_type = 'text/calendar';
+        break;
+      case 'BEGIN:VCARD':
+        dbg_error_log( 'LOG WARNING', 'Application sent content-type of "%s" instead of "text/vcard"',
+                                        (isset($this->content_type)?$this->content_type:'(null)') );
+        $this->content_type = 'text/vcard';
+        break;
+      default:
+        dbg_error_log( 'LOG NOTICE', 'Unusual content-type of "%s" and first word of content is "%s"',
+                                        (isset($this->content_type)?$this->content_type:'(null)'), $first_word );
+    }
+  }
+
+
   /**
   * Returns true if the URL referenced by this request points at a collection.
   */
@@ -1124,7 +1178,7 @@ EOSQL;
     @header( sprintf("X-DAViCal-Version: DAViCal/%d.%d.%d; DB/%d.%d.%d", $c->code_major, $c->code_minor, $c->code_patch, $c->schema_major, $c->schema_minor, $c->schema_patch) );
     @header( "Content-type: ".$content_type );
 
-    if ( (isset($c->dbg['ALL']) && $c->dbg['ALL']) || (isset($c->dbg['response']) && $c->dbg['response']) ) {
+    if ( (isset($c->dbg['ALL']) && $c->dbg['ALL']) || (isset($c->dbg['response']) && $c->dbg['response']) || $status > 399 ) {
       $lines = headers_list();
       dbg_error_log( "LOG ", "***************** Response Header ****************" );
       foreach( $lines AS $v ) {
