@@ -259,7 +259,15 @@ class DAVResource
     else {
       $this->resourcetypes = '';
       if ( isset($this->resource->caldav_data) ) {
-        if ( substr($this->resource->caldav_data,0,15) == 'BEGIN:VCALENDAR' ) $this->contenttype = 'text/calendar';
+        if ( substr($this->resource->caldav_data,0,15) == 'BEGIN:VCALENDAR' ) {
+          $this->contenttype = 'text/calendar';
+        }
+        else if ( strtoupper(substr($this->resource->caldav_data,0,11)) == 'BEGIN:VCARD' ) {
+          $this->contenttype = 'text/vcard';
+        }
+        else if ( strtoupper(substr($this->resource->caldav_data,0,11)) == 'BEGIN:VLIST' ) {
+          $this->contenttype = 'text/x-vlist';
+        }
         $this->resource->displayname = $this->resource->summary;
       }
     }
@@ -479,7 +487,7 @@ EOSQL;
     $this->principal = new CalDAVPrincipal( array( "path" => $this->bound_from() ) );
     if ( $this->_is_principal && $this->principal->Exists() ) {
       $this->exists = true;
-      $this->unique_tag = $this->principal->dav_etag;
+      $this->unique_tag = '"'.$this->principal->dav_etag.'"';
       $this->created = $this->principal->created;
       $this->modified = $this->principal->modified;
       $this->resourcetypes = '<DAV::collection/><DAV::principal/>';
@@ -505,7 +513,9 @@ EOSQL;
     if ( $this->_is_collection ) return;   // We have all we're going to read
 
     $sql = <<<EOQRY
-SELECT * FROM caldav_data LEFT JOIN calendar_item USING (collection_id,dav_id)
+SELECT calendar_item.*, addressbook_resource.*, caldav_data.*
+     FROM caldav_data LEFT JOIN calendar_item USING (collection_id,dav_id)
+                       LEFT JOIN addressbook_resource USING (dav_id)
      WHERE caldav_data.dav_name = :dav_name
 EOQRY;
     $params = array( ':dav_name' => $this->bound_from() );
@@ -513,15 +523,8 @@ EOQRY;
     $qry = new AwlQuery( $sql, $params );
     if ( $qry->Exec('DAVResource') && $qry->rows() > 0 ) {
       $this->exists = true;
-      $this->resource = $qry->Fetch();
-      $this->unique_tag = $this->resource->dav_etag;
-      $this->created = $this->resource->created;
-      $this->modified = $this->resource->modified;
-      $this->resource_id = $this->resource->dav_id;
-      if ( substr($this->resource->caldav_data,0,15) == 'BEGIN:VCALENDAR' ) {
-        $this->contenttype = 'text/calendar';
-      }
-      $this->resourcetypes = '';
+      $row = $qry->Fetch();
+      $this->FromRow($row);
     }
     else {
       $this->exists = false;
@@ -634,13 +637,21 @@ EOQRY;
   /**
   * Is the user has the privileges to do what is requested.
   * @param $do_what mixed The request privilege name, or array of privilege names, to be checked.
+  * @param $any boolean Whether we accept any of the privileges. The default is true, unless the requested privilege is 'all', when it is false.
   * @return boolean Whether they do have one of those privileges against this resource.
   */
-  function HavePrivilegeTo( $do_what ) {
+  function HavePrivilegeTo( $do_what, $any = null ) {
     if ( !isset($this->privileges) ) $this->FetchPrivileges();
+    if ( !isset($any) ) $any = ($do_what != 'all');
     $test_bits = privilege_to_bits( $do_what );
-    dbg_error_log( 'DAVResource', 'Testing privileges of "%s" (%s) against allowed "%s" => "%s" (%s)', $do_what, decbin($test_bits), decbin($this->privileges), ($this->privileges & $test_bits), decbin($this->privileges & $test_bits) );
-    return ($this->privileges & $test_bits) > 0;
+    dbg_error_log( 'DAVResource', 'Testing %s privileges of "%s" (%s) against allowed "%s" => "%s" (%s)', ($any?'any':'exactly'),
+        $do_what, decbin($test_bits), decbin($this->privileges), ($this->privileges & $test_bits), decbin($this->privileges & $test_bits) );
+    if ( $any ) {
+      return ($this->privileges & $test_bits) > 0;
+    }
+    else {
+      return ($this->privileges & $test_bits) == $test_bits;
+    }
   }
 
 
@@ -649,10 +660,10 @@ EOQRY;
   *
   * @param string $privilege The name of the needed privilege.
   */
-  function NeedPrivilege( $privilege ) {
+  function NeedPrivilege( $privilege, $any = null ) {
     global $request;
 
-    if ( $this->HavePrivilegeTo($privilege) ) return;
+    if ( $this->HavePrivilegeTo($privilege, $any) ) return;
 
     $request->NeedPrivilege( $privilege, $this->dav_name );
     exit(0);  // Unecessary, but might clarify things
@@ -665,7 +676,7 @@ EOQRY;
   function BuildPrivileges( $privilege_names=null, &$xmldoc=null ) {
     if ( $privilege_names == null ) {
       if ( !isset($this->privileges) ) $this->FetchPrivileges();
-      $privilege_names = bits_to_privilege($this->privileges);
+      $privilege_names = bits_to_privilege($this->privileges, ($this->_is_collection ? $this->collection->type : null ) );
     }
     return privileges_to_XML( $privilege_names, $xmldoc);
   }
@@ -783,6 +794,15 @@ EOQRY;
         )
       );
     }
+    if ( $this->collection->is_addressbook ) {
+      $this->supported_reports = array_merge(
+        $this->supported_reports,
+        array(
+          'urn:ietf:params:xml:ns:carddav:addressbook-query' => '',
+          'urn:ietf:params:xml:ns:carddav:addressbook-multiget' => ''
+        )
+      );
+    }
     return $this->supported_reports;
   }
 
@@ -1011,6 +1031,19 @@ EOQRY;
     return $this->dav_name();
   }
 
+
+  /**
+  * Sets the dav_name of the resource we are bound as
+  */
+  function set_bind_location( $new_dav_name ) {
+    if ( !isset($this->bound_from) && isset($this->dav_name) ) {
+      $this->bound_from = $this->dav_name;
+    }
+    $this->dav_name = $new_dav_name;
+    return $this->dav_name;
+  }
+
+
   /**
   * Returns the dav_name of the resource in our internal namespace
   */
@@ -1042,7 +1075,7 @@ EOQRY;
   /**
   * Returns the database row for this resource
   */
-  function event() {
+  function resource() {
     if ( !isset($this->resource) ) $this->FetchResource();
     return $this->resource;
   }
@@ -1122,7 +1155,7 @@ EOQRY;
   * BuildACE - construct an XMLElement subtree for a DAV::ace
   */
   function BuildACE( &$xmldoc, $privs, $principal ) {
-    $privilege_names = bits_to_privilege($privs);
+    $privilege_names = bits_to_privilege($privs, ($this->_is_collection ? $this->collection->type : 'resource'));
     $privileges = array();
     foreach( $privilege_names AS $k ) {
       $privilege = new XMLElement('privilege');
@@ -1274,26 +1307,33 @@ EOQRY;
         break;
 
       case 'DAV::parent-set':
-        $parent_set = $reply->DAVElement( $prop, 'parent-set' );
-        if ( preg_match( '{^(.*)/([^/]+)/?$}', $this->bound_from(), $matches ) ) {
-          $reply->DAVElement($parent_set, 'parent', array(
-                              new XMLElement( 'href', ConstructURL($matches[1])),
-                              new XMLElement( 'segment', $matches[2])
-                            ));
-          $sql = <<<EOQRY
-SELECT regexp_replace( b.parent_container, '/$', '') AS parent,
-       regexp_replace( replace( b.dav_name, b.parent_container, ''), '/$', '') AS segment
-  FROM dav_binding b JOIN collection c ON(b.bound_source_id=c.collection_id)
+        $sql = <<<EOQRY
+SELECT b.parent_container FROM dav_binding b JOIN collection c ON (b.bound_source_id=c.collection_id)
  WHERE regexp_replace( b.dav_name, '^.*/', c.dav_name ) = :bound_from
 EOQRY;
-          $qry = new AwlQuery($sql, array( ':bound_from' => $this->bound_from ) );
-          if ( $qry->Exec('DAVResource',__LINE__,__FILE__) && $qry->rows() > 0 ) {
-            while( $row = $qry->Fetch() ) {
-              $reply->DAVElement($parent_set, 'parent', array(
-                                  new XMLElement( 'href', ConstructURL($row->parent)),
-                                  new XMLElement( 'segment', $row->segment)
-                                ));
-            }
+        $qry = new AwlQuery($sql, array( ':bound_from' => $this->bound_from() ) );
+        $parents = array();
+        if ( $qry->Exec('DAVResource',__LINE__,__FILE__) && $qry->rows() > 0 ) {
+          while( $row = $qry->Fetch() ) {
+            $parents[$row->parent_container] = true;
+          }
+        }
+        $parents[preg_replace( '{(?<=/)[^/]+/?$}','',$this->bound_from())] = true;
+        $parents[preg_replace( '{(?<=/)[^/]+/?$}','',$this->dav_name())] = true;
+
+        $parent_set = $reply->DAVElement( $prop, 'parent-set' );
+        foreach( $parents AS $parent => $v ) {
+          if ( preg_match( '{^(.*)?/([^/]+)/?$}', $parent, $matches ) ) {
+            $reply->DAVElement($parent_set, 'parent', array(
+                                new XMLElement( 'href', ConstructURL($matches[1])),
+                                new XMLElement( 'segment', $matches[2])
+                              ));
+          }
+          else if ( $parent == '/' ) {
+            $reply->DAVElement($parent_set, 'parent', array(
+                                new XMLElement( 'href', '/'),
+                                new XMLElement( 'segment', ( ConstructURL('/') == '/caldav.php/' ? 'caldav.php' : ''))
+                              ));
           }
         }
         break;
@@ -1451,6 +1491,7 @@ EOQRY;
         $reply->NSElement($prop, $tag, $cal->Render() );
         break;
 
+      case 'urn:ietf:params:xml:ns:carddav:address-data':
       case 'urn:ietf:params:xml:ns:caldav:calendar-data':
         if ( $this->_is_collection ) return false;
         if ( !isset($this->resource) ) $this->FetchResource();
