@@ -65,6 +65,14 @@ class DAViCalSession extends Session
   * @param object $u The user+session object we (probably) read from the database.
   */
   function AssignSessionDetails( $u ) {
+    if ( !isset($u->principal_id) ) {
+      // If they don't have a principal_id set then we should re-read from our local database
+      $qry = new AwlQuery('SELECT * FROM dav_principal WHERE username = :username', array(':username' => $u->username) );
+      if ( $qry->Exec() && $qry->rows() == 1 ) {
+        $u = $qry->Fetch();
+      }
+    }
+
     parent::AssignSessionDetails( $u );
     $this->GetRoles();
     if ( function_exists('awl_set_locale') && isset($this->locale) && $this->locale != '' ) {
@@ -109,6 +117,16 @@ class DAViCalSession extends Session
     }
     if ( isset($c->restrict_admin_roles) && $roles == '' ) $roles = $c->restrict_admin_roles;
     if ( $this->logged_in && $roles == '' ) return;
+
+    /**
+     * We allow basic auth to apply also, if present, though we check everything else first...
+     */
+    if ( !$this->logged_in && $_SERVER['PHP_AUTH_USER'] != "" && $_SERVER['PHP_AUTH_PW'] != "" && ! $_COOKIE['NoAutoLogin'] ) {
+      if ( $this->Login($_SERVER['PHP_AUTH_USER'],$_SERVER['PHP_AUTH_PW'],true)) {
+        setcookie('NoAutoLogin',1,0);
+        return;
+      }
+    }
     if ( ! $this->logged_in ) {
       $c->messages[] = i18n('You must log in to use this system.');
       include_once('page-header.php');
@@ -129,7 +147,7 @@ class DAViCalSession extends Session
       }
     }
     else {
-      $valid_roles = split(',', $roles);
+      $valid_roles = explode(',', $roles);
       foreach( $valid_roles AS $k => $v ) {
         if ( $this->AllowedTo($v) ) return;
       }
