@@ -37,42 +37,34 @@ $GLOBALS['debug_rrule'] = false;
 * Wrap the DateTimeZone class to allow parsing some iCalendar TZID strangenesses
 */
 class RepeatRuleTimeZone extends DateTimeZone {
-  private $tzid;
+  private $tz_defined;
 
   public function __construct($dtz = null) {
-    $this->tzid = false;
+    $this->tz_defined = false;
     if ( !isset($dtz) ) return;
 
+    $dtz = olson_from_tzstring($dtz);
+
     try {
       parent::__construct($dtz);
-      $this->tzid = $dtz;
+      $this->tz_defined = $dtz;
     }
     catch (Exception $e) {
       $original = $dtz;
-      $dtz = olson_from_tzstring($dtz);
-      if ( isset($dtz) ) {
-        try {
-          parent::__construct($dtz);
-          $this->tzid = $dtz;
-        }
-        catch (Exception $e) {
-          dbg_error_log( 'ERROR', 'Could not parse timezone "%s" - will use floating time', $original );
-          $dtz = new DateTimeZone('UTC');
-          $this->tzid = false;
-        }
-      }
-      else {
+
+      if ( !isset($dtz) ) {
         dbg_error_log( 'ERROR', 'Could not parse timezone "%s" - will use floating time', $original );
         $dtz = new DateTimeZone('UTC');
-        $this->tzid = false;
+        $this->tz_defined = false;
       }
     }
   }
 
   function tzid() {
-    $tzid = parent::getName();
+    if ( $this->tz_defined === false ) return false;
+    $tzid = $this->getName();
     if ( $tzid != 'UTC' ) return $tzid;
-    return $this->tzid;
+    return $this->tz_defined;
   }
 }
 
@@ -91,6 +83,8 @@ class RepeatRuleDateTime extends DateTime {
 
   public function __construct($date = null, $dtz = null) {
     $this->is_date = false;
+    if ( !isset($date) ) return;
+
     if ( preg_match('{;?VALUE=DATE[:;]}', $date, $matches) ) $this->is_date = true;
     elseif ( preg_match('{:([12]\d{3}) (0[1-9]|1[012]) (0[1-9]|[12]\d|3[01]Z?) $}x', $date, $matches) ) $this->is_date = true;
     if (preg_match('/;?TZID=([^:]+).*:(\d{8}(T\d{6})?)(Z)?/', $date, $matches) ) {
@@ -107,11 +101,7 @@ class RepeatRuleDateTime extends DateTime {
         $this->tzid = null;
       }
     }
-    elseif ( is_string($dtz) ) {
-      $dtz = new RepeatRuleTimeZone($dtz);
-      $this->tzid = $dtz->tzid();
-    }
-    elseif( $dtz === null ) {
+    elseif( $dtz === null || $dtz == '' ) {
       $dtz = new RepeatRuleTimeZone('UTC');
       if ( preg_match('/(\d{8}(T\d{6})?)Z/', $date, $matches) ) {
         if ( strlen($matches[1]) == 8 ) $this->is_date = true;
@@ -121,6 +111,10 @@ class RepeatRuleDateTime extends DateTime {
         $this->tzid = null;
       }
     }
+    elseif ( is_string($dtz) ) {
+      $dtz = new RepeatRuleTimeZone($dtz);
+      $this->tzid = $dtz->tzid();
+    }
     else {
       $this->tzid = $dtz->getName();
     }
@@ -141,6 +135,24 @@ class RepeatRuleDateTime extends DateTime {
   }
 
 
+  public function modify( $interval ) {
+    if ( preg_match('{^(-)?P((\d+)W)?((\d+)D)?T?((\d+)H)?((\d+)M)?((\d+)S)?$}', $interval, $matches) ) {
+      $minus = $matches[1];
+      $interval = '';
+      if ( isset($matches[2]) && $matches[2] != '' ) $interval .= $minus . $matches[3] . ' weeks ';
+      if ( isset($matches[4]) && $matches[4] != '' ) $interval .= $minus . $matches[5] . ' days ';
+      if ( isset($matches[6]) && $matches[6] != '' ) $interval .= $minus . $matches[7] . ' hours ';
+      if ( isset($matches[8]) && $matches[8] != '' ) $interval .= $minus . $matches[9] . ' minutes ';
+      if (isset($matches[10]) &&$matches[10] != '' ) $interval .= $minus . $matches[11] . ' seconds ';
+    }
+//    printf( "Modify '%s' by: >>%s<<\n", $this->__toString(), $interval );
+//    print_r($this);
+    if ( !isset($interval) || $interval == '' ) $interval = '1 day';
+    parent::modify($interval);
+    return $this->__toString();
+  }
+
+
   public function UTC() {
     $gmt = clone($this);
     if ( isset($this->tzid) && $this->tzid != 'UTC' ) {
@@ -171,6 +183,11 @@ class RepeatRuleDateTime extends DateTime {
   }
 
 
+  public function RFC5545Duration( $end_stamp ) {
+    return sprintf( 'PT%dM', intval(($end_stamp->epoch() - $this->epoch()) / 60) );
+  }
+
+
   public function setTimeZone( $tz ) {
     if ( is_string($tz) ) {
       $tz = new RepeatRuleTimeZone($tz);
@@ -414,7 +431,7 @@ class RepeatRule {
     $instances = $this->current_set;
     $this->current_set = array();
     foreach( $instances AS $k => $instance ) {
-      foreach( $this->bymonth AS $k => $monthday ) {
+      foreach( $this->bymonthday AS $k => $monthday ) {
         $this->current_set[] = $this->date_mask( clone($instance), null, null, $monthday, null, null, null);
       }
     }
@@ -640,3 +657,178 @@ class RepeatRule {
 
 
 }
+
+
+require_once("vComponent.php");
+
+/**
+* Expand the event instances for an RDATE or EXDATE property
+*
+* @param string $property RDATE or EXDATE, depending...
+* @param array $component A vComponent which is a VEVENT, VTODO or VJOURNAL
+* @param array $range_end A date after which we care less about expansion
+*
+* @return array An array keyed on the UTC dates, referring to the component
+*/
+function rdate_expand( $dtstart, $property, $component, $range_end = null ) {
+  $properties = $component->GetProperties($property);
+  $expansion = array();
+  foreach( $properties AS $p ) {
+    $timezone = $p->GetParameterValue('TZID');
+    $rdate = $p->Value();
+    $rdates = explode( ',', $rdate );
+    foreach( $rdates AS $k => $v ) {
+      $rdate = new RepeatRuleDateTime( $v, $timezone);
+      $expansion[$rdate->UTC()] = $component;
+      if ( $rdate > $range_end ) break;
+    }
+  }
+  return $expansion;
+}
+
+
+/**
+* Expand the event instances for an RRULE property
+*
+* @param object $dtstart A RepeatRuleDateTime which is the master dtstart
+* @param string $property RDATE or EXDATE, depending...
+* @param array $component A vComponent which is a VEVENT, VTODO or VJOURNAL
+* @param array $range_end A date after which we care less about expansion
+*
+* @return array An array keyed on the UTC dates, referring to the component
+*/
+function rrule_expand( $dtstart, $property, $component, $range_end ) {
+  $expansion = array();
+
+  $recur = $component->GetProperty($property);
+  if ( !isset($recur) ) return $expansion;
+  $recur = $recur->Value();
+
+  $this_start = $component->GetProperty('DTSTART');
+  if ( isset($this_start) ) {
+    $timezone = $this_start->GetParameterValue('TZID');
+    $this_start = new RepeatRuleDateTime($this_start->Value(),$timezone);
+  }
+  else {
+    $this_start = clone($dtstart);
+  }
+
+//  print_r( $this_start );
+//  printf( "RRULE: %s\n", $recur );
+  $rule = new RepeatRule( $this_start, $recur );
+  $i = 0;
+  $result_limit = 1000;
+  while( $date = $rule->next() ) {
+//    printf( "[%3d] %s\n", $i, $date->UTC() );
+    $expansion[$date->UTC()] = $component;
+    if ( $i++ >= $result_limit || $date > $range_end ) break;
+  }
+//  print_r( $expansion );
+  return $expansion;
+}
+
+
+/**
+* Expand the event instances for an iCalendar VEVENT (or VTODO)
+*
+* @param object $vResource A vComponent which is a VCALENDAR containing components needing expansion
+* @param object $range_start A RepeatRuleDateTime which is the beginning of the range for events, default -6 weeks
+* @param object $range_end A RepeatRuleDateTime which is the end of the range for events, default +6 weeks
+*
+* @return vComponent The original vComponent, with the instances of the internal components expanded.
+*/
+function expand_event_instances( $vResource, $range_start = null, $range_end = null ) {
+  $components = $vResource->GetComponents();
+
+  if ( !isset($range_start) ) { $range_start = new RepeatRuleDateTime(); $range_start->modify('-6 weeks'); }
+  if ( !isset($range_end) )   { $range_end   = clone($range_start);      $range_end->modify('+6 months');  }
+
+  $new_components = array();
+  $result_limit = 1000;
+  $instances = array();
+  $expand = false;
+  $dtstart = null;
+  foreach( $components AS $k => $comp ) {
+    if ( $comp->GetType() != 'VEVENT' && $comp->GetType() != 'VTODO' && $comp->GetType() != 'VJOURNAL' ) {
+      $new_components[] = $comp;
+      continue;
+    }
+    if ( !isset($dtstart) ) {
+      $dtstart = $comp->GetProperty('DTSTART');
+      $tzid = $dtstart->GetParameterValue('TZID');
+      $dtstart = new RepeatRuleDateTime( $dtstart->Value(), $tzid );
+      $instances[$dtstart->UTC()] = $comp;
+    }
+    $p = $comp->GetProperty('RECURRENCE-ID');
+    if ( isset($p) && $p->Value() != '' ) {
+      $range = $p->GetParameterValue('RANGE');
+      $recur_tzid = $p->GetParameterValue('TZID');
+      $recur_utc = new RepeatRuleDateTime($p->Value(),$recur_tzid);
+      $recur_utc = $recur_utc->UTC();
+      if ( isset($range) && $range == 'THISANDFUTURE' ) {
+        foreach( $instances AS $k => $v ) {
+          if ( $k >= $recur_utc ) unset($instances[$k]);
+        }
+      }
+      else {
+        unset($instances[$recur_utc]);
+      }
+      $instances[] = $comp;
+    }
+    $instances = array_merge( $instances, rrule_expand($dtstart, 'RRULE', $comp, $range_end) );
+    $instances = array_merge( $instances, rdate_expand($dtstart, 'RDATE', $comp, $range_end) );
+    foreach ( rdate_expand($dtstart, 'EXDATE', $comp, $range_end) AS $k => $v ) {
+      unset($instances[$k]);
+    }
+  }
+
+  $last_duration = null;
+  $in_range = false;
+  $new_components = array();
+  $start_utc = $range_start->UTC();
+  $end_utc = $range_end->UTC();
+  foreach( $instances AS $utc => $comp ) {
+    if ( $utc > $end_utc ) break;
+
+    $end_type = ($comp->GetType() == 'VTODO' ? 'DUE' : 'DTEND');
+    $duration = $comp->GetProperty('DURATION');
+    if ( !isset($duration) || $duration->Value() == '' ) {
+      $instance_start = $comp->GetProperty('DTSTART');
+      $dtsrt = new RepeatRuleDateTime( $instance_start->Value(), $instance_start->GetParameterValue('TZID'));
+      $instance_end = $comp->GetProperty($end_type);
+      $dtend = new RepeatRuleDateTime( $instance_end->Value(), $instance_end->GetParameterValue('TZID'));
+      $duration = $dtstart->RFC5545Duration( $dtend );
+    }
+    else {
+      $duration = $duration->Value();
+    }
+
+    if ( $utc < $start_utc ) {
+      if ( isset($last_duration) && $duration == $last_duration) {
+        if ( $utc < $early_start ) continue;
+      }
+      else {
+        $latest_start = clone($range_start);
+        $latest_start->modify('-'.$duration);
+        $early_start = $latest_start->UTC();
+        $last_duration = $duration;
+        if ( $utc < $early_start ) continue;
+      }
+    }
+    $component = clone($comp);
+    $component->ClearProperties( array('DTSTART'=> true, 'DUE' => true, 'DTEND' => true) );
+    $component->AddProperty('DTSTART', $utc );
+    $component->AddProperty('DURATION', $duration );
+    $new_components[] = $component;
+    $in_range = true;
+  }
+
+  if ( $in_range ) {
+    $vResource->SetComponents($new_components);
+  }
+  else {
+    $vResource->SetComponents(array());
+  }
+
+  return $vResource;
+}
