@@ -140,8 +140,6 @@ VALUES( :user_no, :parent_container, :dav_name, :dav_etag, :dav_displayname, TRU
 */
 function public_events_only( $user_no, $dav_name ) {
   global $c;
-  // Not supported until DB versions from 1.001.010
-  if ( $c->schema_version < 1001.010 ) return false;
 
   $sql = 'SELECT public_events_only FROM collection WHERE dav_name = :dav_name';
 
@@ -386,7 +384,7 @@ function update_scheduling_requests( &$resource ) {
 *
 * Any VEVENTs with the same UID will be concatenated together
 */
-function import_collection( $ics_content, $user_no, $path, $caldav_context ) {
+function import_collection( $ics_content, $user_no, $path, $caldav_context, $appending = false ) {
   global $c, $session, $tz_regex;
 
   if ( ! ini_get('open_basedir') && (isset($c->dbg['ALL']) || isset($c->dbg['put'])) ) {
@@ -402,7 +400,7 @@ function import_collection( $ics_content, $user_no, $path, $caldav_context ) {
   $components = $calendar->GetComponents('VTIMEZONE',false);
 
   $displayname = $calendar->GetPValue('X-WR-CALNAME');
-  if ( isset($displayname) ) {
+  if ( !$appending && isset($displayname) ) {
     $sql = 'UPDATE collection SET dav_displayname = :displayname WHERE dav_name = :dav_name';
     $qry = new AwlQuery( $sql, array( ':displayname' => $displayname, ':dav_name' => $path) );
     if ( ! $qry->Exec('PUT',__LINE__,__FILE__) ) rollback_on_error( $caldav_context, $user_no, $path );
@@ -441,9 +439,11 @@ function import_collection( $ics_content, $user_no, $path, $caldav_context ) {
 
   if ( !(isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import) ) $qry->Begin();
   $base_params = array( ':collection_id' => $collection->collection_id );
-  if ( !$qry->QDo('DELETE FROM calendar_item WHERE collection_id = :collection_id', $base_params)
+  if ( !$appending ) {
+    if ( !$qry->QDo('DELETE FROM calendar_item WHERE collection_id = :collection_id', $base_params)
       || !$qry->QDo('DELETE FROM caldav_data WHERE collection_id = :collection_id', $base_params) )
-    rollback_on_error( $caldav_context, $user_no, $collection->collection_id );
+      rollback_on_error( $caldav_context, $user_no, $collection->collection_id );
+  }
 
   $dav_data_insert = <<<EOSQL
 INSERT INTO caldav_data ( user_no, dav_name, dav_etag, caldav_data, caldav_type, logged_user, created, modified, collection_id )
@@ -845,7 +845,7 @@ function write_resource( $user_no, $path, $caldav_data, $collection_id, $author,
 
 
   /** Calculate what timezone to set, first, if possible */
-  $last_tz_locn = 'Turkmenikikamukau';
+  $last_tz_locn = 'Turkmenikikamukau';  // I really hope this location doesn't exist!
   $tzid = $first->GetPParamValue('DTSTART','TZID');
   if ( !isset($tzid) || $tzid == '' ) $tzid = $first->GetPParamValue('DUE','TZID');
   $timezones = $ic->GetComponents('VTIMEZONE');
@@ -855,7 +855,7 @@ function write_resource( $user_no, $path, $caldav_data, $collection_id, $author,
       * We'll pretend they didn't forget to give us a TZID and that they
       * really hope the server is running in the timezone they supplied... but be noisy about it.
       */
-      dbg_error_log( 'ERROR', ' Event includes TZID[%s] but users TZID[%s]!', $tz->GetPValue('TZID'), $tzid );
+      dbg_error_log( 'ERROR', ' Event includes TZID[%s] but uses TZID[%s]!', $tz->GetPValue('TZID'), $tzid );
       $tzid = $tz->GetPValue('TZID');
     }
     // This is the one
@@ -863,7 +863,7 @@ function write_resource( $user_no, $path, $caldav_data, $collection_id, $author,
     if ( ! isset($tz_locn) ) {
       if ( preg_match( '#([^/]+/[^/]+)$#', $tzid, $matches ) )
         $tz_locn = $matches[1];
-      else {
+      else if ( isset($tzid) && $tzid != '' ) {
         dbg_error_log( 'ERROR', ' Couldn\'t guess Olsen TZ from TZID[%s].  This may end in tears...', $tzid );
       }
     }
@@ -938,7 +938,11 @@ EOSQL;
   if ( $log_action && function_exists('log_caldav_action') ) {
     log_caldav_action( $put_action_type, $first->GetPValue('UID'), $user_no, $collection_id, $path );
   }
-
+  else if ( $log_action  ) {
+    dbg_error_log( 'PUT', 'No log_caldav_action( %s, %s, %s, %s, %s) can be called.',
+            $put_action_type, $first->GetPValue('UID'), $user_no, $collection_id, $path );
+  }
+  
   $qry = new AwlQuery( $sql, $calitem_params );
   if ( !$qry->Exec('PUT',__LINE__,__FILE__) ) {
     rollback_on_error( $caldav_context, $user_no, $path);
@@ -963,7 +967,7 @@ EOSQL;
 * @param string $put_action_type INSERT or UPDATE depending on what we are to do
 * @return boolean True for success, false for failure.
 */
-function simple_write_resource( $path, $caldav_data, $put_action_type ) {
+function simple_write_resource( $path, $caldav_data, $put_action_type, $write_action_log = false ) {
 
   $etag = md5($caldav_data);
   $ic = new iCalComponent( $caldav_data );
@@ -977,7 +981,7 @@ function simple_write_resource( $path, $caldav_data, $put_action_type ) {
     $collection = $qry->Fetch();
     $user_no = $collection->user_no;
 
-    return write_resource( $user_no, $path, $caldav_data, $collection->collection_id, $user_no, $etag, $ic, $put_action_type, false, false );
+    return write_resource( $user_no, $path, $caldav_data, $collection->collection_id, $user_no, $etag, $ic, $put_action_type, false, $write_action_log );
   }
   return false;
 }
