@@ -9,6 +9,13 @@ $responses = array();
 $searches = $xmltree->GetPath('/DAV::principal-property-search/DAV::property-search');
 dbg_log_array( "principal", "SEARCH", $searches, true );
 
+$clause_joiner = " AND ";
+$CS_search_test = $xmltree->GetAttribute('test');
+if ( isset($CS_search_test) && $CS_search_test == 'anyof' ) {
+  $clause_joiner = " OR ";
+}
+
+$params = array();
 $where = "";
 foreach( $searches AS $k => $search ) {
   $qry_props = $search->GetPath('/DAV::property-search/DAV::prop/*');  // There may be many
@@ -20,10 +27,21 @@ foreach( $searches AS $k => $search ) {
     if ( $subwhere != "" ) $subwhere .= " OR ";
     switch( $v1->GetTag() ) {
       case 'DAV::displayname':
-        $subwhere .= "username = ".qpg($match);
+        $subwhere .= ' displayname ILIKE :displayname_match ';
+        $params[':displayname_match'] = '%'.$match.'%';
         break;
+
+      case 'urn:ietf:params:xml:ns:caldav:calendar-user-address-set':
+        $match = preg_replace('{^.*/caldav.php/([^/]+)(/.*)?$}', '\\1', $match);
+        $match = preg_replace('{^mailto:}', '', $match);
+        $subwhere .= ' (email ILIKE :user_address_match OR username ILIKE :user_address_match) ';
+        $params[':user_address_match'] = '%'.$match.'%';
+        break;
+
       case 'urn:ietf:params:xml:ns:caldav:calendar-home-set':
-        $subwhere .= "username = ".qpg(preg_replace('#^.*/caldav.php/([^/]+)(/|$)#', "\\1", $match));
+        $match = preg_replace('{^.*/caldav.php}', '', $match);
+        $subwhere .= ' dav_name LIKE :calendar_home_match ';
+        $params[':calendar_home_match'] = $match.'%';
         break;
 
       default:
@@ -35,12 +53,12 @@ foreach( $searches AS $k => $search ) {
     }
   }
   if ( $subwhere != "" ) {
-    $where .= sprintf( "%s(%s)", ($where == "" ? "" : " AND "), $subwhere );
+    $where .= sprintf( "%s(%s)", ($where == "" ? "" : $clause_joiner), $subwhere );
   }
 }
 if ( $where != "" ) $where = "WHERE $where";
-$sql = "SELECT * FROM usr $where";
-$qry = new PgQuery($sql);
+$sql = "SELECT * FROM dav_principal $where ORDER BY principal_id LIMIT 100";
+$qry = new AwlQuery($sql, $params);
 
 
 $get_props = $xmltree->GetPath('/DAV::principal-property-search/DAV::prop/*');
@@ -49,9 +67,9 @@ foreach( $get_props AS $k1 => $v1 ) {
   $properties[] = $v1->GetTag();
 }
 
-if ( $qry->Exec("REPORT",__LINE__,__FILE__) && $qry->rows > 0 ) {
+if ( $qry->Exec("REPORT",__LINE__,__FILE__) && $qry->rows() > 0 ) {
   while( $row = $qry->Fetch() ) {
-    $principal = new CalDAVPrincipal($row);
+    $principal = new DAVResource($row);
     $responses[] = $principal->RenderAsXML( $properties, $reply );
   }
 }
