@@ -246,9 +246,14 @@ class CalDAVClient {
     $this->request_url = $url;
     $url = preg_replace('{^https?://[^/]+}', '', $url);
     // URLencode if it isn't already
-/*    if ( !preg_match( '{(%\x\x)}', $url) && preg_match( '{[^.-_/a-z0-9]}', $url ) ) {
+    if ( preg_match( '{[^%?&=+,.-_/a-z0-9]}', $url ) ) {
       $url = str_replace(rawurlencode('/'),'/',rawurlencode($url));
-    }*/
+      $url = str_replace(rawurlencode('?'),'?',$url);
+      $url = str_replace(rawurlencode('&'),'&',$url);
+      $url = str_replace(rawurlencode('='),'=',$url);
+      $url = str_replace(rawurlencode('+'),'+',$url);
+      $url = str_replace(rawurlencode(','),',',$url);
+    }
     $headers[] = $this->requestMethod." ". $url . " HTTP/1.1";
     $headers[] = "Authorization: Basic ".base64_encode($this->user .":". $this->pass );
     $headers[] = "Host: ".$this->server .":".$this->port;
@@ -273,22 +278,46 @@ class CalDAVClient {
     while( !feof($fip) ) { $response .= fgets($fip,8192); }
     fclose($fip);
 
-    $pos = strpos($response, "\n\n");
-    if ( $pos === false ) {
-      $this->httpResponseHeaders = $response;
-      $this->httpResponseBody = '';
-    }
-    else {
-      $this->httpResponseHeaders = substr($response,0,pos+1);
-      $this->httpResponseBody = substr($response, $pos + 2);
-    }
+    list( $this->httpResponseHeaders, $this->httpResponseBody ) = preg_split( '{\r?\n\r?\n}s', $response, 2 );
+    if ( preg_match( '{Transfer-Encoding: chunked}i', $this->httpResponseHeaders ) ) $this->Unchunk();
 
     $this->headers = array();  // reset the headers array for our next request
-    $this->ParseResponse($response);
+    $this->ParseResponse($this->httpResponseBody);
     return $response;
   }
 
 
+  /**
+  * Unchunk a chunked response
+  */
+  function Unchunk() {
+    $content = '';
+    $chunks = $this->httpResponseBody;
+    // printf( "\n================================\n%s\n================================\n", $chunks );
+    do {
+      $bytes = 0;
+      if ( preg_match('{^((\r\n)?\s*([ 0-9a-fA-F]+)(;[^\n]*)?\r?\n)}', $chunks, $matches ) ) {
+        $octets = $matches[3];
+        $bytes = hexdec($octets);
+        $pos = strlen($matches[1]);
+        // printf( "Chunk size 0x%s (%d)\n", $octets, $bytes );
+        if ( $bytes > 0 ) {
+          // printf( "---------------------------------\n%s\n---------------------------------\n", substr($chunks,$pos,$bytes) );
+          $content .= substr($chunks,$pos,$bytes);
+          $chunks = substr($chunks,$pos + $bytes + 2);
+          // printf( "+++++++++++++++++++++++++++++++++\n%s\n+++++++++++++++++++++++++++++++++\n", $chunks );
+        }
+      }
+      else {
+        $content .= $chunks;
+      }
+    }
+    while( $bytes > 0 );
+    $this->httpResponseBody = $content;
+    // printf( "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n%s\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n", $content );
+  }
+
+
   /**
   * Send an OPTIONS request to the server
   *
@@ -365,18 +394,22 @@ class CalDAVClient {
     if ( $etag != null ) {
       $this->SetMatch( ($etag != '*'), $etag );
     }
-    $this->SetContentType('text/icalendar; encoding="utf-8"');
+    $this->SetContentType('text/calendar; encoding="utf-8"');
     $this->DoRequest($url);
 
     $etag = null;
     if ( preg_match( '{^ETag:\s+"([^"]*)"\s*$}im', $this->httpResponseHeaders, $matches ) ) $etag = $matches[1];
     if ( !isset($etag) || $etag == '' ) {
       printf( "No etag in:\n%s\n", $this->httpResponseHeaders );
+      $save_request = $this->httpRequest;
+      $save_response_headers = $this->httpResponseHeaders;
       $this->DoHEADRequest( $url );
       if ( preg_match( '{^Etag:\s+"([^"]*)"\s*$}im', $this->httpResponseHeaders, $matches ) ) $etag = $matches[1];
       if ( !isset($etag) || $etag == '' ) {
         printf( "Still No etag in:\n%s\n", $this->httpResponseHeaders );
       }
+      $this->httpRequest = $save_request;
+      $this->httpResponseHeaders = $save_response_headers;
     }
     return $etag;
   }
@@ -547,6 +580,9 @@ class CalDAVClient {
           if ( $status == 'HTTP/1.1 200 OK' ) break;
         }
       }
+      elseif ( !isset($this->xmlnodes[$nodenum]) || !is_array($this->xmlnodes[$nodenum]) ) {
+        break;
+      }
       elseif ( $this->xmlnodes[$nodenum]['tag'] == 'DAV::status' ) {
         $status = $this->xmlnodes[$nodenum]['value'];
       }
