@@ -139,6 +139,7 @@ class ldapDrivers
       for($i = ldap_first_entry($this->connect,$entry);
           $i && $arr = ldap_get_attributes($this->connect,$i);
           $i = ldap_next_entry($this->connect,$i) ) {
+        $row = array();
         for ($j=0; $j < $arr['count']; $j++) {
           $row[$arr[$j]] = $arr[$arr[$j]][0];
         }
@@ -147,7 +148,7 @@ class ldapDrivers
     }
     return $ret;
   }
-  
+
   /**
   * Retrieve all groups from the LDAP directory
   */
@@ -161,7 +162,7 @@ class ldapDrivers
 
       if (!ldap_first_entry($this->connect,$entry)) {
         $c->messages[] = sprintf(translate('Error NoGroupFound with filter >%s<, attributes >%s< , dn >%s<'),
-                                 $this->filterUsers,
+                                 $this->filterGroups,
                                  join(', ', $attributes),
                                  $baseDNGroups);
       }
@@ -211,7 +212,7 @@ class ldapDrivers
 
     $dnUser = ldap_get_dn($this->connect, ldap_first_entry($this->connect,$entry));
 
-    if ($c->authenticate_hook['config']['i_use_mode_kerberos'] == "i_know_what_i_am_doing") {
+    if ( isset($c->authenticate_hook['config']['i_use_mode_kerberos']) && $c->authenticate_hook['config']['i_use_mode_kerberos'] == "i_know_what_i_am_doing") {
     	dbg_error_log( "LOG", "drivers_ldap : Skipping password Check for user %s which should be the same as %s",$username , $_SERVER["REMOTE_USER"]);
       if ($username != $_SERVER["REMOTE_USER"]) {
         return false;
@@ -363,13 +364,18 @@ function sync_LDAP_groups(){
     //$attributes = array('cn','modifyTimestamp','memberUid');
     $attributes = array_values($mapping);
     $ldap_groups_tmp = $ldapDriver->getAllGroups($attributes);
-    
+
     if ( sizeof($ldap_groups_tmp) == 0 )
       return;
 
     foreach($ldap_groups_tmp as $key => $ldap_group){
       $ldap_groups_info[$ldap_group[$mapping['username']]] = $ldap_group;
+      if (is_array($ldap_groups_info[$ldap_group[$mapping['username']]][$mapping['members']])) {
       unset ( $ldap_groups_info[$ldap_group[$mapping['username']]][$mapping['members']]['count'] );
+      }
+      else {
+          $ldap_groups_info[$ldap_group[$mapping['username']]][$mapping['members']] = array($ldap_groups_info[$ldap_group[$mapping['username']]][$mapping['members']]);
+      }
       unset($ldap_groups_tmp[$key]);
     }
     $db_groups = array ();
@@ -380,7 +386,7 @@ function sync_LDAP_groups(){
       $db_groups[$db_group->group_name] = $db_group->group_name;
       $db_group_members[$db_group->group_name][] = $db_group->member_name;
     }
-    
+
     $ldap_groups = array_keys($ldap_groups_info);
     // users only in ldap
     $groups_to_create = array_diff($ldap_groups,$db_groups);
@@ -388,13 +394,13 @@ function sync_LDAP_groups(){
     $groups_to_deactivate = array_diff($db_groups,$ldap_groups);
     // users present in ldap and in the db
     $groups_to_update = array_intersect($db_groups,$ldap_groups);
-    
+
     if ( sizeof ( $groups_to_create ) ){
       $c->messages[] = sprintf(i18n('- creating groups : %s'),join(', ',$groups_to_create));
       $validUserFields = get_fields('usr');
       foreach ( $groups_to_create as $k => $group ){
         $user = (object) array( 'user_no' => 0, 'username' => '' );
-    
+
         if ( isset($c->authenticate_hook['config']['default_value']) && is_array($c->authenticate_hook['config']['default_value']) ) {
           foreach ( $c->authenticate_hook['config']['default_value'] as $field => $value ) {
             if ( isset($validUserFields[$field]) ) {
@@ -404,6 +410,7 @@ function sync_LDAP_groups(){
           }
         }
         $user->user_no = 0;
+        $ldap_values = $ldap_groups_info[$group];
         foreach ( $mapping as $field => $value ) {
           dbg_error_log( "LDAP", "Considering copying %s", $field );
           if ( isset($validUserFields[$field]) ) {
@@ -411,9 +418,15 @@ function sync_LDAP_groups(){
             dbg_error_log( "LDAP", "Setting usr->%s to %s from LDAP field %s", $field, $ldap_values[$value], $value );
           }
         }
-        $user->fullname = $group;  
-        $user->displayname = $group;  
-        $user->username = $group;  
+        if ($user->fullname=="") {
+          $user->fullname = $group;
+        }
+        if ($user->displayname=="") {
+          $user->displayname = $group;
+        }
+        $user->username = $group;
+        $user->updated = "now";  /** @todo Use the 'updated' timestamp from LDAP for groups too */
+
         UpdateUserFromExternal( $user );
         $qry = new AwlQuery( "UPDATE dav_principal set type_id = 3 WHERE username=:group ",array(':group'=>$group) );
         $qry->Exec('sync_LDAP',__LINE__,__FILE__);
