@@ -22,8 +22,13 @@ $delete_collection_confirmation_required = null;
 $delete_principal_confirmation_required = null;
 
 function handle_subaction( $subaction ) {
-  global $session, $c, $id, $editor, $delete_collection_confirmation_required, $delete_principal_confirmation_required;
-
+  global $session, $c, $id, $editor;
+  global $delete_collection_confirmation_required;
+  global $delete_principal_confirmation_required;
+  global $delete_ticket_confirmation_required;
+  global $delete_bind_in_confirmation_required;
+  global $delete_binding_confirmation_required;
+  
   dbg_error_log('admin-principal-edit',':handle_action: Action %s', $subaction );
 
   switch( $subaction ) {
@@ -54,9 +59,9 @@ function handle_subaction( $subaction ) {
     case 'delete_principal':
       dbg_error_log('admin-principal-edit',':handle_action: Deleting principal %d', $id );
       if ( $session->AllowedTo('Admin') ) {
-        if ( $session->CheckConfirmationHash('GET', 'confirm') ) {
+        if ( isset($id) && $id > 1 && $session->CheckConfirmationHash('GET', 'confirm') ) {
           dbg_error_log('admin-principal-edit',':handle_action: Allowed to delete principal %d -%s', $id, $editor->Value('username') );
-          $qry = new AwlQuery('DELETE FROM dav_principal WHERE principal_id=?;', $id );
+          $qry = new AwlQuery('DELETE FROM dav_principal WHERE principal_id=?', $id );
           if ( $qry->Exec() ) {
             $c->messages[] = i18n('Principal deleted');
             return true;
@@ -74,7 +79,61 @@ function handle_subaction( $subaction ) {
       }
       break;
 
-    default:
+    case 'delete_ticket':
+      dbg_error_log('admin-principal-edit',':handle_action: Deleting ticket "%s" for principal %d', $_GET['ticket_id'], $id );
+      if ( $session->AllowedTo('Admin')
+                || ($id > 0 && $session->principal_id == $id) ) {
+        if ( $session->CheckConfirmationHash('GET', 'confirm') ) {
+          dbg_error_log('admin-principal-edit',':handle_action: Allowed to delete ticket "%s" for principal %d', $_GET['ticket_id'], $id );
+          $qry = new AwlQuery('DELETE FROM access_ticket WHERE ticket_id=?;', $_GET['ticket_id'] );
+          if ( $qry->Exec() ) {
+            $c->messages[] = i18n('Access ticket deleted');
+            return true;
+          }
+          else {
+            $c->messages[] = i18n('There was an error writing to the database.');
+            return false;
+          }
+        }
+        else {
+          $c->messages[] = i18n('Please confirm deletion of access ticket - see below');
+          $delete_ticket_confirmation_required = $session->BuildConfirmationHash('GET', 'confirm');
+          return false;
+        }
+      }
+      break;
+
+    case 'delete_bind_in':
+    case 'delete_binding':
+      dbg_error_log('admin-principal-edit',':handle_action: Deleting binding "%s" for principal %d', $_GET['bind_id'], $id );
+      if ( $session->AllowedTo('Admin')
+                || ($id > 0 && $session->principal_id == $id) ) {
+        if ( $session->CheckConfirmationHash('GET', 'confirm') ) {
+          dbg_error_log('admin-principal-edit',':handle_action: Allowed to delete ticket "%s" for principal %d', $_GET['bind_id'], $id );
+          $qry = new AwlQuery('DELETE FROM dav_binding WHERE bind_id=?;', $_GET['bind_id'] );
+          if ( $qry->Exec() ) {
+            $c->messages[] = i18n('Binding deleted');
+            return true;
+          }
+          else {
+            $c->messages[] = i18n('There was an error writing to the database.');
+            return false;
+          }
+        }
+        else {
+          $c->messages[] = i18n('Please confirm deletion of binding - see below');
+          if ( $subaction == 'delete_bind_in' ) {
+            $delete_bind_in_confirmation_required = $session->BuildConfirmationHash('GET', 'confirm');
+          }
+          else {
+            $delete_binding_confirmation_required = $session->BuildConfirmationHash('GET', 'confirm');
+          }
+          return false;
+        }
+      }
+      break;
+
+      default:
       return false;
   }
   return false;
@@ -155,7 +214,7 @@ if ( $can_write_principal && $editor->IsSubmit() ) {
     $qry->Exec('admin-principal-edit');
   }
 }
-else {
+else if ( isset($id) && $id > 0 ) {
   $editor->GetRecord();
   if ( $editor->IsSubmit() ) {
     $c->messages[] = i18n('You do not have permission to modify this record.');
@@ -370,7 +429,7 @@ if ( isset($delete_principal_confirmation_required) ) {
 }
 
 
-if ( isset($id) ) {
+if ( isset($id) && $id > 0 ) {
   $browser = new Browser(translate('Group Memberships'));
   $c->stylesheets[] = 'css/browse.css';
   $c->scripts[] = 'js/browse.js';
@@ -614,6 +673,39 @@ EOTEMPLATE;
   }
 
 
+  $browser = new Browser(translate('Access Tickets'));
+
+  $browser->AddColumn( 'ticket_id', translate('Ticket ID'), '', '' );
+  $browser->AddColumn( 'target', translate('Target'), '', '<td style="white-space:nowrap;">%s</td>', "'".$c->base_url.'/caldav.php'."' ||COALESCE(d.dav_name,c.dav_name)" );
+  $browser->AddColumn( 'expiry', translate('Expires'), '', '', 'TO_CHAR(expires,\'YYYY-MM-DD HH:MI:SS\')');
+  $browser->AddColumn( 'privs', translate('Privileges'), '', '', "privileges_list(privileges)" );
+  $delurl = $c->base_url . '/admin.php?action=edit&t=principal&id='.$id.'&ticket_id=##URL:ticket_id##&subaction=delete_ticket';
+  $browser->AddColumn( 'delete', translate('Action'), 'center', '', "'<a class=\"submit\" href=\"$delurl\">".translate('Delete')."</a>'" );
+
+  $browser->SetOrdering( 'target', 'A' );
+
+  $browser->SetJoins( 'access_ticket t LEFT JOIN collection c ON (target_collection_id=collection_id) LEFT JOIN caldav_data d ON (target_resource_id=dav_id)' );
+  $browser->SetWhere( 'dav_owner_id = '.intval($editor->Value('principal_id')) );
+
+//  $browser->AddRow( array( 'target' => '<a href="'.$rowurl.'&principal_id='.intval($editor->Value('principal_id')).'" class="submit">'.translate('Create Ticket').'</a>' ));
+
+  $browser->RowFormat( '<tr class="r%d">', '</tr>', '#even' );
+
+  $browser->DoQuery();
+  $page_elements[] = $browser;
+
+  if ( isset($delete_ticket_confirmation_required) ) {
+    $html = '<table><tr><td class="error">';
+    $html .= sprintf('<b>%s</b> "%s" <a class="error" href="%s&%s">%s</a> %s',
+                translate('Deleting Ticket:'), $_GET['ticket_id'], $_SERVER['REQUEST_URI'],
+                $delete_ticket_confirmation_required,
+                translate('Confirm Deletion of the Ticket'),
+                translate('The access ticket will be deleted.') );
+    $html .= "</td></tr></table>\n";
+    $page_elements[] = $html;
+  }
+
+
   $browser = new Browser(translate('Principal Collections'));
 
   $browser->AddColumn( 'collection_id', translate('ID'), 'right', '##collection_link##' );
@@ -653,5 +745,69 @@ EOTEMPLATE;
     $page_elements[] = $html;
   }
 
+  $browser = new Browser(translate('Bindings to other collections'));
+  $browser->AddColumn( 'bind_id', translate('ID'), '', '' );
+  $browser->AddHidden( 'b.dav_owner_id' );
+  $browser->AddHidden( 'p.principal_id' );
+  $browser->AddColumn( 'dav_name', translate('Collection'), '', '<td style="white-space:nowrap;">%s</td>', 'c.dav_name' );
+  $browser->AddColumn( 'bound_as', translate('Bound As'), '', '<td style="white-space:nowrap;">%s</td>', 'b.dav_name' );
+  $browser->AddColumn( 'access_ticket_id', translate('Ticket ID'), '', '' );
+  $browser->AddColumn( 'privs', translate('Privileges'), '', '', "privileges_list(privileges)" );
+  $delurl = $c->base_url . sprintf('/admin.php?action=edit&t=principal&id=%s&bind_id=##bind_id##&subaction=delete_bind_in', $editor->Value('principal_id'));
+  $browser->AddColumn( 'delete', translate('Action'), 'center', '', "'<a class=\"submit\" href=\"$delurl\">".translate('Delete')."</a>'" );
 
+  $browser->SetOrdering( 'target', 'A' );
+
+  $browser->SetJoins( 'dav_binding b LEFT JOIN collection c ON (bound_source_id=collection_id) LEFT JOIN access_ticket t ON (ticket_id=access_ticket_id) LEFT JOIN principal p USING(user_no)' );
+  $browser->SetWhere( 'b.dav_name ~ '.sprintf("'^/%s/'", $editor->Value('username')) );
+
+  $browser->RowFormat( '<tr class="r%d">', '</tr>', '#even' );
+
+  $browser->DoQuery();
+  $page_elements[] = $browser;
+
+  if ( isset($delete_bind_in_confirmation_required) ) {
+    $html = '<table><tr><td class="error">';
+    $html .= sprintf('<b>%s</b> "%s" <a class="error" href="%s&%s">%s</a> %s',
+                translate('Deleting Binding:'), $_GET['bind_id'], $_SERVER['REQUEST_URI'],
+                $delete_bind_in_confirmation_required,
+                translate('Confirm Deletion of the Binding'),
+                translate('The binding will be deleted.') );
+    $html .= "</td></tr></table>\n";
+    $page_elements[] = $html;
+  }
+  
+  
+  $browser = new Browser(translate('Bindings to this Principal\'s Collections'));
+  $browser->AddColumn( 'bind_id', translate('ID'), '', '' );
+  $browser->AddHidden( 'b.dav_owner_id' );
+  $browser->AddHidden( 'p.principal_id' );
+  $browser->AddColumn( 'dav_name', translate('Collection'), '', '<td style="white-space:nowrap;">%s</td>', 'c.dav_name' );
+  $browser->AddColumn( 'bound_as', translate('Bound As'), '', '<td style="white-space:nowrap;">%s</td>', 'b.dav_name' );
+  $browser->AddColumn( 'access_ticket_id', translate('Ticket ID'), '', '' );
+  $browser->AddColumn( 'privs', translate('Privileges'), '', '', "privileges_list(privileges)" );
+  $delurl = $c->base_url . '/admin.php?action=edit&t=principal&id=##principal_id##&bind_id=##bind_id##&subaction=delete_binding';
+  $browser->AddColumn( 'delete', translate('Action'), 'center', '', "'<a class=\"submit\" href=\"$delurl\">".translate('Delete')."</a>'" );
+
+  $browser->SetOrdering( 'target', 'A' );
+
+  $browser->SetJoins( 'dav_binding b LEFT JOIN collection c ON (bound_source_id=collection_id) LEFT JOIN access_ticket t ON (ticket_id=access_ticket_id) LEFT JOIN principal p USING(user_no)' );
+  $browser->SetWhere( 'p.principal_id = '.intval($editor->Value('principal_id')) );
+
+  $browser->RowFormat( '<tr class="r%d">', '</tr>', '#even' );
+
+  $browser->DoQuery();
+  $page_elements[] = $browser;
+
+  if ( isset($delete_binding_confirmation_required) ) {
+    $html = '<table><tr><td class="error">';
+    $html .= sprintf('<b>%s</b> "%s" <a class="error" href="%s&%s">%s</a> %s',
+                translate('Deleting Binding:'), $_GET['bind_id'], $_SERVER['REQUEST_URI'],
+                $delete_binding_confirmation_required,
+                translate('Confirm Deletion of the Binding'),
+                translate('The binding will be deleted.') );
+    $html .= "</td></tr></table>\n";
+    $page_elements[] = $html;
+  }
+  
 }
