@@ -16,12 +16,12 @@ export PGTZ=Pacific/Auckland
 [ -n "${ALTHOST}"  ] && ALTHOST="--althost ${ALTHOST}"
 
 
-UNTIL=${1:-"99999"}
+SUITE=${1:-"regression-suite"}
 ACCEPT_ALL=${2:-""}
-
+[ -z "${UNTIL}" ] && UNTIL=99999
 [ -z "${SUITE}" ] && SUITE="regression-suite"
-REGRESSION="tests/${SUITE}"
-RESULTS="${REGRESSION}/results"
+
+
 
 check_result() {
   TEST="$1"
@@ -85,48 +85,93 @@ drop_database() {
   fi
 }
 
-drop_database ${DBNAME}
 
-mkdir -p "${RESULTS}"
-mkdir -p "${REGRESSION}/diffs"
+restore_database() {
+  drop_database ${DBNAME}
 
-TEST="Create-Database"
-../dba/create-database.sh ${DBNAME} 'nimda' >"${RESULTS}/${TEST}" 2>&1
-check_result "${TEST}"
+  TEST="Restore-Database"
+  createdb --owner davical_dba --encoding UTF8 ${DBNAME} >"${RESULTS}/${TEST}" 2>&1
+  psql ${DBNAME} -q -f "${REGRESSION}/initial.dbdump" >>"${RESULTS}/${TEST}" 2>&1
+  check_result "${TEST}"
+}
+
+
+dump_database() {
+  TEST="Dump-Database"
+  pg_dump -Fp ${DBNAME} > "${REGRESSION}/initial.dbdump" 2>&1
+}
+
+
+initialise_regression() {
+  drop_database ${DBNAME}
+
+  TEST="Create-Database"
+  ../dba/create-database.sh ${DBNAME} 'nimda' >"${RESULTS}/${TEST}" 2>&1
+  check_result "${TEST}"
+
+  TEST="Upgrade-Database"
+  ../dba/update-davical-database --dbname=${DBNAME} --nopatch --appuser davical_app --owner davical_dba >"${RESULTS}/${TEST}" 2>&1
+  check_result "${TEST}"
+
+  TEST="Load-Sample-Data"
+  psql -q -f "../dba/sample-data.sql" "${DBNAME}" >"${RESULTS}/${TEST}" 2>&1
+  check_result "${TEST}"
+}
+
+
+run_regression_suite() {
+  RESULTS="${REGRESSION}/results"
+  mkdir -p "${RESULTS}"
+  mkdir -p "${REGRESSION}/diffs"
+
+  if [ -f "${REGRESSION}/initial.dbdump" ]; then
+    restore_database
+  else
+    initialise_regression
+  fi
+
+  for T in ${REGRESSION}/*.test ; do
+    TEST="`basename ${T} .test`"
+    TESTNUM="`echo ${TEST} | cut -f1 -d'-'`"
+    TESTNUM="${TEST/-*}"
+    if [ "${TESTNUM}" -gt "${UNTIL}" ] ; then
+      break;
+    fi
+
+    RESULT=999
+    while [ "${RESULT}" -gt 0 ]; do
+      ./dav_test --dsn "${DSN}" ${WEBHOST} ${ALTHOST} --suite "${SUITE}" --case "${TEST}" | ./normalise_result > "${RESULTS}/${TEST}"
+      # Fix Vim syntax highlighting by putting an esac here.  Silly, huh?
+
+      RESULT=999
+      while [ "${RESULT}" -gt 1 ]; do
+        check_result "${TEST}"
+        RESULT=$?
+      done
+
+    done
+
+    TCOUNT="$(( ${TCOUNT} + 1 ))"
+  done
+}
 
-TEST="Upgrade-Database"
-../dba/update-davical-database --dbname=${DBNAME} --nopatch --appuser davical_app --owner davical_dba >"${RESULTS}/${TEST}" 2>&1
-check_result "${TEST}"
 
-TEST="Load-Sample-Data"
-psql -q -f "../dba/sample-data.sql" "${DBNAME}" >"${RESULTS}/${TEST}" 2>&1
-check_result "${TEST}"
 
 TSTART="`date +%s`"
 TCOUNT=0
 
-for T in ${REGRESSION}/*.test ; do
-  TEST="`basename ${T} .test`"
-  TESTNUM="`echo ${TEST} | cut -f1 -d'-'`"
-  TESTNUM="${TEST/-*}"
-  if [ "${TESTNUM}" -gt "${UNTIL}" ] ; then
-    break;
-  fi
-
-  RESULT=999
-  while [ "${RESULT}" -gt 0 ]; do
-    ./dav_test --dsn "${DSN}" ${WEBHOST} ${ALTHOST} --suite "${SUITE}" --case "${TEST}" | ./normalise_result > "${RESULTS}/${TEST}"
-
-    RESULT=999
-    while [ "${RESULT}" -gt 1 ]; do
-      check_result "${TEST}"
-      RESULT=$?
-    done
-
+if [ "${SUITE}" = "all" ]; then
+  for SUITE in regression-suite binding carddav ; do
+    REGRESSION="tests/${SUITE}"
+    if [ "${SUITE}" != "regression-suite" ]; then
+      dump_database
+    fi
+    run_regression_suite "${SUITE}"
   done
-
-  TCOUNT="$(( ${TCOUNT} + 1 ))"
-done
+else
+  REGRESSION="tests/${SUITE}"
+  run_regression_suite "${SUITE}"
+fi
 TFINISH="`date +%s`"
 
 echo "Regression test run took $(( ${TFINISH} - ${TSTART} )) seconds for ${TCOUNT} tests."
